import ipaddress

from django.db import models
from django.core.exceptions import ValidationError

from .lookups import IpIn
from .lookups import IpContains


class IPNetworkField(models.Field):
    description = 'A Django database field that is compatible with IPv4 and IPv6 and supports CIDR.'

    def db_type(self, connection):
        return 'char(150)'

    def get_internal_type(self):
        return 'CharField'

    def to_python(self, value):
        if isinstance(value, str):
            value = value.strip()
        try:
            return ipaddress.ip_network(value, strict=False)
        except ValueError as error:
            raise ValidationError(error)

    def from_db_value(self, value, expression, connection):
        if value is None or len(value) < 5:
            return None
        prefix = value[:4]
        value = value[4:-1]    # remove prefix and %
        if prefix == 'IPv4':
            max_length = ipaddress.IPV4LENGTH
        else:
            max_length = ipaddress.IPV6LENGTH
        string_ip = ipaddress.ip_address(int(value + '0' * (max_length-len(value)), 2)).__str__()
        string_network = string_ip + '/{}'.format(len(value))
        return ipaddress.ip_network(string_network)

    def get_prep_value(self, value):
        try:
            value = self.to_python(value)
        except ValidationError:
            return None
        if isinstance(value, ipaddress.IPv4Network):
            max_length = ipaddress.IPV4LENGTH
            prefix = 'IPv4'
        elif isinstance(value, ipaddress.IPv6Network):
            max_length = ipaddress.IPV6LENGTH
            prefix = 'IPv6'
        else:
            return str(value)
        bin_ip = str(bin(int(value.network_address)))[2:]
        bin_ip = '0' * (max_length - len(bin_ip)) + bin_ip
        bin_network = prefix + bin_ip[:value.prefixlen] + '%'
        return bin_network

    def get_prep_lookup(self, lookup_type, value):
        return self.get_prep_value(value)

    def get_lookup(self, lookup_name):
        if lookup_name == 'contains' or lookup_name == 'icontains':
            return IpContains
        elif lookup_name == 'in':
            return IpIn
        return super().get_lookup(lookup_name)



from setuptools import setup, find_packages
import sys, os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import cidrfield

setup(
    name="django-cidrfield",
    version=cidrfield.__version__,
    url="https://github.com/Werneror/django-cidrfield",
    author="Werner",
    author_email="me@werner.wiki",
    license="MIT",
    description="Proper CIDR fields for Django running on any database",
    long_description=open('README.rst').read(),
    keywords="ip, cidr, models, django",
    packages=["cidrfield"],
    setup_requires=["setuptools"],
    install_requires=("setuptools", "django",),
    classifiers=(
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",),
)




from django.test import TestCase
from django.db.models import Model
from django.db import IntegrityError
from .models import IPNetworkField

import ipaddress


class DummyModel(Model):
    field = IPNetworkField()
    null_field = IPNetworkField(null=True)

    def __str__(self):
        return field.__str__()

    __repr__ = __str__


class IPNetworkFieldTests(TestCase):

    def test_ipv4_equal(self):
        DummyModel.objects.create(field='192.168.1.0/24')
        self.assertEqual(DummyModel.objects.filter(field='192.168.1.0/24').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field='192.168.1.32').count(), 0)
        self.assertEqual(DummyModel.objects.filter(field='192.168.2.0/24').count(), 0)

    def test_ipv6_equal(self):
        DummyModel.objects.create(field='FEC0:0000:0000:0000:0000:0000:0000:0000/10')
        self.assertEqual(DummyModel.objects.filter(field='fec0::/10').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field='fec0::1').count(), 0)
        self.assertEqual(DummyModel.objects.filter(field='2001:4860:4860::8888').count(), 0)

    def test_ipv4_in(self):
        DummyModel.objects.create(field='10.10.10.0/24')
        self.assertEqual(DummyModel.objects.filter(field__in='10.10.0.0/16').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field__in='10.10.10.0/30').count(), 0)

    def test_ipv6_in(self):
        DummyModel.objects.create(field='fec4::/16')
        self.assertEqual(DummyModel.objects.filter(field__in='fec0::/10').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field__in='fec0::1').count(), 0)

    def test_ipv4_contains(self):
        DummyModel.objects.create(field='10.20.10.0/24')
        self.assertEqual(DummyModel.objects.filter(field__contains='10.20.10.1').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field__icontains='10.20.10.1').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field__contains='10.30.10.1').count(), 0)
        self.assertEqual(DummyModel.objects.filter(field__icontains='10.30.10.1').count(), 0)

    def test_ipv6_contains(self):
        DummyModel.objects.create(field='fec8::/16')
        self.assertEqual(DummyModel.objects.filter(field__contains='fec8::1').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field__icontains='fec8::1').count(), 1)
        self.assertEqual(DummyModel.objects.filter(field__contains='fec4::1').count(), 0)
        self.assertEqual(DummyModel.objects.filter(field__icontains='fec4::1').count(), 0)

    def test_required_values(self):
        with self.assertRaises(IntegrityError):
            # non-null field should require value
            DummyModel.objects.create()

    def test_null_values(self):
        # null field can be left empty
        DummyModel.objects.create(field='::1')

    def test_max_values(self):
        DummyModel.objects.create(field='ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')
        DummyModel.objects.create(field='255.255.255.255')



from django.db.models import Lookup


class IpContains(Lookup):
    lookup_name = 'contains'

    def as_sql(self, compiler, connection):
        lhs, lhs_params = self.process_lhs(compiler, connection)
        rhs, rhs_params = self.process_rhs(compiler, connection)
        params = rhs_params + lhs_params
        return '%s like %s' % (rhs, lhs), params


class IpIn(Lookup):
    lookup_name = 'in'

    def as_sql(self, compiler, connection):
        lhs, lhs_params = self.process_lhs(compiler, connection)
        rhs, rhs_params = self.process_rhs(compiler, connection)
        params = lhs_params + rhs_params
        return '%s like %s' % (lhs, rhs), params
